@@ -6,7 +6,7 @@ use panic_halt as _;
 use cortex_m_rt::entry;
 use stm32f4xx_hal as hal;
 
-use crate::hal::{pac, prelude::*, serial::config::Config, serial::Serial};
+use crate::hal::{pac, prelude::*, serial::Serial};
 
 use core::fmt::Write; // for pretty formatting of the serial output
 
@@ -27,20 +27,14 @@ fn main() -> ! {
     let tx_pin = gpioa.pa2.into_alternate();
 
     // configure serial
-    let mut tx = Serial::tx(
-        dp.USART2,
-        tx_pin,
-        Config::default().baudrate(9600.bps()),
-        clocks,
-    )
-    .unwrap();
+    let mut tx = Serial::tx(dp.USART2, tx_pin, 9600.bps(), &clocks).unwrap();
 
     let mut value: u8 = 0;
 
     loop {
         // print some value every 500 ms, value will overflow after 255
         writeln!(tx, "value: {:02}\r", value).unwrap();
-        value += 1;
+        value = value.wrapping_add(1);
         delay.delay_ms(500_u32);
     }
 }
