@@ -12,7 +12,6 @@ use crate::dma::traits::PeriAddress;
 use crate::rcc::{Enable, Reset};
 use crate::{gpio::*, pac, signature::VrefCal, signature::VDDA_CALIB};
 use core::fmt;
-use embedded_hal::adc::{Channel, OneShot};
 
 /// Vref internal signal, used for calibration
 pub struct Vref;
@@ -26,7 +25,7 @@ pub struct Temperature;
 macro_rules! adc_pins {
     ($($pin:ty => ($adc:ident, $chan:expr)),+ $(,)*) => {
         $(
-            impl Channel<pac::$adc> for $pin {
+            impl embedded_hal::adc::Channel<pac::$adc> for $pin {
                 type ID = u8;
                 fn channel() -> u8 { $chan }
             }
@@ -605,7 +604,7 @@ pub struct Adc<ADC> {
     adc_reg: ADC,
     /// VDDA in millivolts calculated from the factory calibration and vrefint
     calibrated_vdda: u32,
-    /// Maximum sample value possible for the configured resolution
+    /// Exclusive limit for the sample value possible for the configured resolution.
     max_sample: u32,
 }
 impl<ADC> fmt::Debug for Adc<ADC> {
@@ -794,10 +793,10 @@ macro_rules! adc {
                 /// Sets the sampling resolution
                 pub fn set_resolution(&mut self, resolution: config::Resolution) {
                     self.max_sample = match resolution {
-                        config::Resolution::Twelve => (1 << 12) - 1,
-                        config::Resolution::Ten => (1 << 10) - 1,
-                        config::Resolution::Eight => (1 << 8) - 1,
-                        config::Resolution::Six => (1 << 6) -1,
+                        config::Resolution::Twelve => (1 << 12),
+                        config::Resolution::Ten => (1 << 10),
+                        config::Resolution::Eight => (1 << 8),
+                        config::Resolution::Six => (1 << 6),
                     };
                     self.config.resolution = resolution;
                     self.adc_reg.cr1.modify(|_, w| w.res().bits(resolution.into()));
@@ -897,7 +896,7 @@ macro_rules! adc {
                 /// to sample for at a given ADC clock frequency
                 pub fn configure_channel<CHANNEL>(&mut self, _channel: &CHANNEL, sequence: config::Sequence, sample_time: config::SampleTime)
                 where
-                    CHANNEL: Channel<pac::$adc_type, ID=u8>
+                    CHANNEL: embedded_hal::adc::Channel<pac::$adc_type, ID=u8>
                 {
                     //Check the sequence is long enough
                     self.adc_reg.sqr1.modify(|r, w| {
@@ -954,7 +953,9 @@ macro_rules! adc {
                     self.adc_reg.dr.read().data().bits()
                 }
 
-                /// Converts a sample value to millivolts using calibrated VDDA and configured resolution
+                /// Converts a sample value to millivolts using calibrated VDDA and configured resolution.
+                /// Due to the ADC characteristics VDDA will never be reached as described in #362 and
+                /// [AN2834-How to get the best ADC accuracy in STM32 microcontrollers](https://www.st.com/resource/en/application_note/cd00211314-how-to-get-the-best-adc-accuracy-in-stm32-microcontrollers-stmicroelectronics.pdf) in section 3.1.2.
                 pub fn sample_to_millivolts(&self, sample: u16) -> u16 {
                     ((u32::from(sample) * self.calibrated_vdda) / self.max_sample) as u16
                 }
@@ -975,7 +976,7 @@ macro_rules! adc {
                 /// Note that it reconfigures the adc sequence and doesn't restore it
                 pub fn convert<PIN>(&mut self, pin: &PIN, sample_time: config::SampleTime) -> u16
                 where
-                    PIN: Channel<pac::$adc_type, ID=u8>
+                    PIN: embedded_hal::adc::Channel<pac::$adc_type, ID=u8>
                 {
                     self.adc_reg.cr2.modify(|_, w| w
                         .dma().clear_bit() //Disable dma
@@ -1006,13 +1007,10 @@ macro_rules! adc {
                 }
             }
 
-            impl<PIN> OneShot<pac::$adc_type, u16, PIN> for Adc<pac::$adc_type>
-            where
-                PIN: Channel<pac::$adc_type, ID=u8>,
-            {
-                type Error = ();
-
-                fn read(&mut self, pin: &mut PIN) -> nb::Result<u16, Self::Error> {
+            impl Adc<pac::$adc_type> {
+                fn read<PIN>(&mut self, pin: &mut PIN) -> nb::Result<u16, ()>
+                    where PIN: embedded_hal::adc::Channel<pac::$adc_type, ID=u8>,
+                {
                     let enabled = self.is_enabled();
                     if !enabled {
                         self.enable();
@@ -1028,6 +1026,17 @@ macro_rules! adc {
                 }
             }
 
+            impl<PIN> embedded_hal::adc::OneShot<pac::$adc_type, u16, PIN> for Adc<pac::$adc_type>
+            where
+                PIN: embedded_hal::adc::Channel<pac::$adc_type, ID=u8>,
+            {
+                type Error = ();
+
+                fn read(&mut self, pin: &mut PIN) -> nb::Result<u16, Self::Error> {
+                    self.read::<PIN>(pin)
+                }
+            }
+
             unsafe impl PeriAddress for Adc<pac::$adc_type> {
                 #[inline(always)]
                 fn address(&self) -> u32 {
