@@ -2,7 +2,7 @@ use super::*;
 use crate::{
     adc::Adc,
     pac::{self, DMA1, DMA2},
-    serial::{Rx, Tx},
+    serial, spi,
 };
 use core::ops::Deref;
 
@@ -346,10 +346,14 @@ dma_map!(
     (Stream7<DMA1>, 5, CCR3<pac::TIM3>, MemoryToPeripheral), //TIM3_CH3
     (Stream7<DMA1>, 5, CCR3<pac::TIM3>, PeripheralToMemory), //TIM3_CH3
     (Stream0<DMA1>, 0, pac::SPI3, PeripheralToMemory),       //SPI3_RX
+    (Stream0<DMA1>, 0, spi::Rx<pac::SPI3>, PeripheralToMemory), //SPI3_RX
     (Stream2<DMA1>, 0, pac::SPI3, PeripheralToMemory),       //SPI3_RX
+    (Stream2<DMA1>, 0, spi::Rx<pac::SPI3>, PeripheralToMemory), //SPI3_RX
     (Stream4<DMA1>, 3, pac::I2C3, MemoryToPeripheral),       //I2C3_TX
     (Stream5<DMA1>, 0, pac::SPI3, MemoryToPeripheral),       //SPI3_TX
+    (Stream5<DMA1>, 0, spi::Tx<pac::SPI3>, MemoryToPeripheral), //SPI3_TX
     (Stream7<DMA1>, 0, pac::SPI3, MemoryToPeripheral),       //SPI3_TX
+    (Stream7<DMA1>, 0, spi::Tx<pac::SPI3>, MemoryToPeripheral), //SPI3_TX
 );
 
 #[cfg(any(
@@ -461,33 +465,37 @@ dma_map!(
     (Stream0<DMA1>, 1, pac::I2C1, PeripheralToMemory),       //I2C1_RX
     (Stream2<DMA1>, 7, pac::I2C2, PeripheralToMemory),       //I2C2_RX
     (Stream3<DMA1>, 0, pac::SPI2, PeripheralToMemory),       //SPI2_RX
+    (Stream3<DMA1>, 0, spi::Rx<pac::SPI2>, PeripheralToMemory), //SPI2_RX
     (Stream3<DMA1>, 7, pac::I2C2, PeripheralToMemory),       //I2C2_RX
     (Stream4<DMA1>, 0, pac::SPI2, MemoryToPeripheral),       //SPI2_TX
+    (Stream4<DMA1>, 0, spi::Tx<pac::SPI2>, MemoryToPeripheral), // SPI2_TX
     (Stream5<DMA1>, 1, pac::I2C1, PeripheralToMemory),       //I2C1_RX
     (Stream5<DMA1>, 4, pac::USART2, PeripheralToMemory),     //USART2_RX
-    (Stream5<DMA1>, 4, Rx<pac::USART2>, PeripheralToMemory), //USART2_RX
+    (Stream5<DMA1>, 4, serial::Rx<pac::USART2>, PeripheralToMemory), //USART2_RX
     (Stream6<DMA1>, 4, pac::USART2, MemoryToPeripheral),     //USART2_TX
-    (Stream6<DMA1>, 4, Tx<pac::USART2>, MemoryToPeripheral), //USART2_TX
+    (Stream6<DMA1>, 4, serial::Tx<pac::USART2>, MemoryToPeripheral), //USART2_TX
     (Stream7<DMA1>, 7, pac::I2C2, MemoryToPeripheral),       //I2C2_TX
     (Stream0<DMA2>, 0, pac::ADC1, PeripheralToMemory),       //ADC1
     (Stream0<DMA2>, 0, Adc<pac::ADC1>, PeripheralToMemory),
     (Stream0<DMA2>, 3, pac::SPI1, PeripheralToMemory), //SPI1_RX
+    (Stream0<DMA2>, 3, spi::Rx<pac::SPI1>, PeripheralToMemory), //SPI1_RX
     (Stream1<DMA2>, 5, pac::USART6, PeripheralToMemory), //USART6_RX
-    (Stream1<DMA2>, 5, Rx<pac::USART6>, PeripheralToMemory), //USART6_RX
+    (Stream1<DMA2>, 5, serial::Rx<pac::USART6>, PeripheralToMemory), //USART6_RX
     (Stream2<DMA2>, 3, pac::SPI1, PeripheralToMemory), //SPI1_RX
+    (Stream2<DMA2>, 3, spi::Rx<pac::SPI1>, PeripheralToMemory), //SPI1_RX
     (Stream2<DMA2>, 4, pac::USART1, PeripheralToMemory), //USART1_RX
-    (Stream2<DMA2>, 4, Rx<pac::USART1>, PeripheralToMemory), //USART1_RX
+    (Stream2<DMA2>, 4, serial::Rx<pac::USART1>, PeripheralToMemory), //USART1_RX
     (Stream2<DMA2>, 5, pac::USART6, PeripheralToMemory), //USART6_RX
-    (Stream2<DMA2>, 5, Rx<pac::USART6>, PeripheralToMemory), //USART6_RX
+    (Stream2<DMA2>, 5, serial::Rx<pac::USART6>, PeripheralToMemory), //USART6_RX
     (Stream4<DMA2>, 0, pac::ADC1, PeripheralToMemory), //ADC1
     (Stream5<DMA2>, 4, pac::USART1, PeripheralToMemory), //USART1_RX
-    (Stream5<DMA2>, 4, Rx<pac::USART1>, PeripheralToMemory), //USART1_RX
+    (Stream5<DMA2>, 4, serial::Rx<pac::USART1>, PeripheralToMemory), //USART1_RX
     (Stream6<DMA2>, 5, pac::USART6, MemoryToPeripheral), //USART6_TX
-    (Stream6<DMA2>, 5, Tx<pac::USART6>, MemoryToPeripheral), //USART6_TX
+    (Stream6<DMA2>, 5, serial::Tx<pac::USART6>, MemoryToPeripheral), //USART6_TX
     (Stream7<DMA2>, 4, pac::USART1, MemoryToPeripheral), //USART1_TX
-    (Stream7<DMA2>, 4, Tx<pac::USART1>, MemoryToPeripheral), //USART1_TX
+    (Stream7<DMA2>, 4, serial::Tx<pac::USART1>, MemoryToPeripheral), //USART1_TX
     (Stream7<DMA2>, 5, pac::USART6, MemoryToPeripheral), //USART6_TX
-    (Stream7<DMA2>, 5, Tx<pac::USART6>, MemoryToPeripheral), //USART6_TX
+    (Stream7<DMA2>, 5, serial::Tx<pac::USART6>, MemoryToPeripheral), //USART6_TX
     (Stream0<DMA2>, 0, MemoryToMemory<u8>, MemoryToMemory<u8>),
     (Stream1<DMA2>, 0, MemoryToMemory<u8>, MemoryToMemory<u8>),
     (Stream2<DMA2>, 0, MemoryToMemory<u8>, MemoryToMemory<u8>),
@@ -609,7 +617,9 @@ dma_map!(
     (Stream6<DMA1>, 1, pac::I2C1, MemoryToPeripheral), //I2C1_TX
     (Stream7<DMA1>, 1, pac::I2C1, MemoryToPeripheral), //I2C1_TX
     (Stream3<DMA2>, 3, pac::SPI1, MemoryToPeripheral), //SPI1_TX
+    (Stream3<DMA2>, 3, spi::Tx<pac::SPI1>, MemoryToPeripheral), //SPI1_TX
     (Stream5<DMA2>, 3, pac::SPI1, MemoryToPeripheral), //SPI1_TX
+    (Stream5<DMA2>, 3, spi::Tx<pac::SPI1>, MemoryToPeripheral), //SPI1_TX
 );
 
 #[cfg(any(
@@ -628,9 +638,13 @@ dma_map!(
 ))]
 dma_map!(
     (Stream0<DMA2>, 4, pac::SPI4, PeripheralToMemory), //SPI4_RX
+    (Stream0<DMA2>, 4, spi::Rx<pac::SPI4>, PeripheralToMemory), //SPI4_RX
     (Stream1<DMA2>, 4, pac::SPI4, MemoryToPeripheral), //SPI4_TX
+    (Stream1<DMA2>, 4, spi::Tx<pac::SPI4>, MemoryToPeripheral), //SPI4_TX
     (Stream3<DMA2>, 5, pac::SPI4, PeripheralToMemory), //SPI4_RX:DMA_CHANNEL_5
+    (Stream3<DMA2>, 5, spi::Rx<pac::SPI4>, PeripheralToMemory), //SPI4_RX:DMA_CHANNEL_5
     (Stream4<DMA2>, 5, pac::SPI4, MemoryToPeripheral), //SPI4_TX:DMA_CHANNEL_5
+    (Stream4<DMA2>, 5, spi::Tx<pac::SPI4>, MemoryToPeripheral), //SPI4_TX:DMA_CHANNEL_5
 );
 
 #[cfg(any(
@@ -740,8 +754,11 @@ dma_map!(
     (Stream7<DMA2>, 7, DMAR<pac::TIM8>, MemoryToPeripheral), //TIM8_COM/TRIG
     (Stream7<DMA2>, 7, DMAR<pac::TIM8>, PeripheralToMemory), //TIM8_COM/TRIG
     (Stream1<DMA1>, 4, pac::USART3, PeripheralToMemory),     //USART3_RX
+    (Stream1<DMA1>, 4, serial::Rx<pac::USART3>, PeripheralToMemory), //USART3_RX
     (Stream3<DMA1>, 4, pac::USART3, MemoryToPeripheral),     //USART3_TX
+    (Stream3<DMA1>, 4, serial::Tx<pac::USART3>, MemoryToPeripheral), //USART3_TX
     (Stream4<DMA1>, 7, pac::USART3, MemoryToPeripheral),     //USART3_TX:DMA_CHANNEL_7
+    (Stream4<DMA1>, 7, serial::Tx<pac::USART3>, MemoryToPeripheral), //USART3_TX:DMA_CHANNEL_7
 );
 
 #[cfg(any(
@@ -973,9 +990,13 @@ dma_map!(
     (Stream7<DMA1>, 1, pac::I2C1, MemoryToPeripheral), //I2C1_TX:DMA_CHANNEL_1
     (Stream7<DMA1>, 6, pac::USART2, PeripheralToMemory), //USART2_RX:DMA_CHANNEL_6
     (Stream2<DMA2>, 2, pac::SPI1, MemoryToPeripheral), //SPI1_TX
+    (Stream2<DMA2>, 2, spi::Tx<pac::SPI1>, MemoryToPeripheral), //SPI1_TX
     (Stream3<DMA2>, 3, pac::SPI1, MemoryToPeripheral), //SPI1_TX:DMA_CHANNEL_3
+    (Stream3<DMA2>, 3, spi::Tx<pac::SPI1>, MemoryToPeripheral), //SPI1_TX:DMA_CHANNEL_3
     (Stream5<DMA2>, 3, pac::SPI1, MemoryToPeripheral), //SPI1_TX:DMA_CHANNEL_3
+    (Stream5<DMA2>, 3, spi::Tx<pac::SPI1>, MemoryToPeripheral), //SPI1_TX:DMA_CHANNEL_3
     (Stream5<DMA2>, 5, pac::SPI5, MemoryToPeripheral), //SPI5_TX:DMA_CHANNEL_5
+    (Stream5<DMA2>, 5, spi::Tx<pac::SPI5>, MemoryToPeripheral), //SPI5_TX:DMA_CHANNEL_5
 );
 
 #[cfg(any(
@@ -993,9 +1014,13 @@ dma_map!(
 ))]
 dma_map!(
     (Stream3<DMA2>, 2, pac::SPI5, PeripheralToMemory), //SPI5_RX
+    (Stream3<DMA2>, 2, spi::Rx<pac::SPI5>, PeripheralToMemory), //SPI5_RX
     (Stream4<DMA2>, 2, pac::SPI5, MemoryToPeripheral), //SPI5_TX
+    (Stream4<DMA2>, 2, spi::Tx<pac::SPI5>, MemoryToPeripheral), //SPI5_TX
     (Stream5<DMA2>, 7, pac::SPI5, PeripheralToMemory), //SPI5_RX:DMA_CHANNEL_7
+    (Stream5<DMA2>, 7, spi::Rx<pac::SPI5>, PeripheralToMemory), //SPI5_RX:DMA_CHANNEL_7
     (Stream6<DMA2>, 7, pac::SPI5, MemoryToPeripheral), //SPI5_TX:DMA_CHANNEL_7
+    (Stream6<DMA2>, 7, spi::Tx<pac::SPI5>, MemoryToPeripheral), //SPI5_TX:DMA_CHANNEL_7
 );
 
 #[cfg(any(
@@ -1019,7 +1044,10 @@ address!((pac::SPI5, dr, u8),);
     feature = "stm32f413",
     feature = "stm32f423",
 ))]
-dma_map!((Stream4<DMA2>, 4, pac::SPI4, PeripheralToMemory),); //SPI4_RX);
+dma_map!(
+    (Stream4<DMA2>, 4, pac::SPI4, PeripheralToMemory), //SPI4_RX);
+    (Stream4<DMA2>, 4, spi::Rx<pac::SPI4>, PeripheralToMemory),
+); //SPI4_RX);
 
 /* TODO: DFSDM support
 #[cfg(feature = "stm32f412")]
@@ -1173,7 +1201,9 @@ address!(
 ))]
 dma_map!(
     (Stream5<DMA2>, 1, pac::SPI6, MemoryToPeripheral), //SPI6_TX
+    (Stream5<DMA2>, 1, spi::Tx<pac::SPI6>, MemoryToPeripheral), //SPI6_TX
     (Stream6<DMA2>, 1, pac::SPI6, PeripheralToMemory), //SPI6_RX
+    (Stream6<DMA2>, 1, spi::Rx<pac::SPI6>, PeripheralToMemory), //SPI6_RX
 );
 
 #[cfg(any(
