@@ -23,8 +23,8 @@
 //! Each GPIO pin can be set to various modes:
 //!
 //! - **Alternate**: Pin mode required when the pin is driven by other peripherals
-//! - **AlternateOD**: Pin mode required when the pin is driven by other peripherals and has open drain
 //! - **Analog**: Analog input to be used with ADC.
+//! - **Dynamic**: Pin mode is selected at runtime. See changing configurations for more details
 //! - Input
 //!     - **PullUp**: Input connected to high with a weak pull up resistor. Will be high when nothing
 //!     is connected
@@ -43,24 +43,50 @@
 //! If you need a more temporary mode change, and can not use the `into_<mode>` functions for
 //! ownership reasons, you can use the closure based `with_<mode>` functions to temporarily change the pin type, do
 //! some output or input, and then have it change back once done.
+//!
+//! ### Dynamic Mode Change
+//! The above mode change methods guarantee that you can only call input functions when the pin is
+//! in input mode, and output when in output modes, but can lead to some issues. Therefore, there
+//! is also a mode where the state is kept track of at runtime, allowing you to change the mode
+//! often, and without problems with ownership, or references, at the cost of some performance and
+//! the risk of runtime errors.
+//!
+//! To make a pin dynamic, use the `into_dynamic` function, and then use the `make_<mode>` functions to
+//! change the mode
 
-use core::convert::Infallible;
 use core::marker::PhantomData;
 
-pub use embedded_hal::digital::v2::PinState;
-use embedded_hal::digital::v2::{
-    InputPin, IoPin, OutputPin, StatefulOutputPin, ToggleableOutputPin,
-};
-
 use crate::pac::EXTI;
 use crate::syscfg::SysCfg;
 
+mod alt;
+pub(crate) use alt::{Const, PinA, SetAlternate};
 mod convert;
-pub(crate) use convert::{Const, SetAlternate, SetAlternateOD};
+use convert::PinMode;
 mod partially_erased;
 pub use partially_erased::{PEPin, PartiallyErasedPin};
 mod erased;
 pub use erased::{EPin, ErasedPin};
+mod dynamic;
+pub use dynamic::{Dynamic, DynamicPin};
+mod hal_02;
+
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+pub enum PinState {
+    /// Low pin state
+    Low,
+    /// High pin state
+    High,
+}
+
+impl From<bool> for PinState {
+    fn from(value: bool) -> Self {
+        match value {
+            false => PinState::Low,
+            true => PinState::High,
+        }
+    }
+}
 
 /// A filler pin type
 pub struct NoPin;
@@ -83,28 +109,7 @@ pub trait PinExt {
 }
 
 /// Some alternate mode (type state)
-pub struct Alternate<const A: u8>;
-
-/// Some alternate mode in open drain configuration (type state)
-pub struct AlternateOD<const A: u8>;
-
-// Compatibility constants
-pub const AF0: u8 = 0;
-pub const AF1: u8 = 1;
-pub const AF2: u8 = 2;
-pub const AF3: u8 = 3;
-pub const AF4: u8 = 4;
-pub const AF5: u8 = 5;
-pub const AF6: u8 = 6;
-pub const AF7: u8 = 7;
-pub const AF8: u8 = 8;
-pub const AF9: u8 = 9;
-pub const AF10: u8 = 10;
-pub const AF11: u8 = 11;
-pub const AF12: u8 = 12;
-pub const AF13: u8 = 13;
-pub const AF14: u8 = 14;
-pub const AF15: u8 = 15;
+pub struct Alternate<Otype, const A: u8>(PhantomData<Otype>);
 
 /// Input mode (type state)
 pub struct Input<MODE> {
@@ -134,6 +139,8 @@ pub struct PushPull;
 /// Analog mode (type state)
 pub struct Analog;
 
+pub type Debugger = Alternate<PushPull, 0>;
+
 /// GPIO Pin speed selection
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Speed {
@@ -302,7 +309,7 @@ impl<MODE, const P: char, const N: u8> Pin<Output<MODE>, P, N> {
 
 impl<const P: char, const N: u8> Pin<Output<OpenDrain>, P, N> {
     /// Enables / disables the internal pull up
-    pub fn internal_pull_up(&mut self, on: bool) {
+    pub fn internal_pull_up(self, on: bool) -> Self {
         let offset = 2 * { N };
         let value = if on { 0b01 } else { 0b00 };
         unsafe {
@@ -310,10 +317,25 @@ impl<const P: char, const N: u8> Pin<Output<OpenDrain>, P, N> {
                 .pupdr
                 .modify(|r, w| w.bits((r.bits() & !(0b11 << offset)) | (value << offset)))
         };
+
+        self
+    }
+
+    /// Enables / disables the internal pull down
+    pub fn internal_pull_down(self, on: bool) -> Self {
+        let offset = 2 * { N };
+        let value = if on { 0b10 } else { 0b00 };
+        unsafe {
+            (*Gpio::<P>::ptr())
+                .pupdr
+                .modify(|r, w| w.bits((r.bits() & !(0b11 << offset)) | (value << offset)))
+        };
+
+        self
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> Pin<Alternate<A>, P, N> {
+impl<const P: char, const N: u8, const A: u8> Pin<Alternate<PushPull, A>, P, N> {
     /// Set pin speed
     pub fn set_speed(self, speed: Speed) -> Self {
         let offset = 2 * { N };
@@ -339,11 +361,24 @@ impl<const P: char, const N: u8, const A: u8> Pin<Alternate<A>, P, N> {
 
         self
     }
+
+    /// Enables / disables the internal pull down
+    pub fn internal_pull_down(self, on: bool) -> Self {
+        let offset = 2 * { N };
+        let value = if on { 0b10 } else { 0b00 };
+        unsafe {
+            (*Gpio::<P>::ptr())
+                .pupdr
+                .modify(|r, w| w.bits((r.bits() & !(0b11 << offset)) | (value << offset)))
+        };
+
+        self
+    }
 }
 
-impl<const P: char, const N: u8, const A: u8> Pin<Alternate<A>, P, N> {
+impl<const P: char, const N: u8, const A: u8> Pin<Alternate<PushPull, A>, P, N> {
     /// Turns pin alternate configuration pin into open drain
-    pub fn set_open_drain(self) -> Pin<AlternateOD<A>, P, N> {
+    pub fn set_open_drain(self) -> Pin<Alternate<OpenDrain, A>, P, N> {
         let offset = { N };
         unsafe {
             (*Gpio::<P>::ptr())
@@ -363,10 +398,6 @@ impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
     pub fn erase_number(self) -> PEPin<MODE, P> {
         PEPin::new(N)
     }
-    #[deprecated(since = "0.10.0", note = "Please use erase_number instead")]
-    pub fn downgrade(self) -> PEPin<MODE, P> {
-        self.erase_number()
-    }
 
     /// Erases the pin number and the port from the type
     ///
@@ -375,10 +406,6 @@ impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
     pub fn erase(self) -> EPin<MODE> {
         EPin::new(P as u8 - b'A', N)
     }
-    #[deprecated(since = "0.10.0", note = "Please use erase instead")]
-    pub fn downgrade2(self) -> EPin<MODE> {
-        self.erase()
-    }
 }
 
 impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
@@ -463,44 +490,6 @@ impl<MODE, const P: char, const N: u8> Pin<Output<MODE>, P, N> {
     }
 }
 
-impl<MODE, const P: char, const N: u8> OutputPin for Pin<Output<MODE>, P, N> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn set_high(&mut self) -> Result<(), Self::Error> {
-        self.set_high();
-        Ok(())
-    }
-
-    #[inline(always)]
-    fn set_low(&mut self) -> Result<(), Self::Error> {
-        self.set_low();
-        Ok(())
-    }
-}
-
-impl<MODE, const P: char, const N: u8> StatefulOutputPin for Pin<Output<MODE>, P, N> {
-    #[inline(always)]
-    fn is_set_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_set_high())
-    }
-
-    #[inline(always)]
-    fn is_set_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_set_low())
-    }
-}
-
-impl<MODE, const P: char, const N: u8> ToggleableOutputPin for Pin<Output<MODE>, P, N> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn toggle(&mut self) -> Result<(), Self::Error> {
-        self.toggle();
-        Ok(())
-    }
-}
-
 impl<const P: char, const N: u8> Pin<Output<OpenDrain>, P, N> {
     #[inline(always)]
     pub fn is_high(&self) -> bool {
@@ -513,20 +502,6 @@ impl<const P: char, const N: u8> Pin<Output<OpenDrain>, P, N> {
     }
 }
 
-impl<const P: char, const N: u8> InputPin for Pin<Output<OpenDrain>, P, N> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn is_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_high())
-    }
-
-    #[inline(always)]
-    fn is_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_low())
-    }
-}
-
 impl<MODE, const P: char, const N: u8> Pin<Input<MODE>, P, N> {
     #[inline(always)]
     pub fn is_high(&self) -> bool {
@@ -539,56 +514,6 @@ impl<MODE, const P: char, const N: u8> Pin<Input<MODE>, P, N> {
     }
 }
 
-impl<MODE, const P: char, const N: u8> InputPin for Pin<Input<MODE>, P, N> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn is_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_high())
-    }
-
-    #[inline(always)]
-    fn is_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_low())
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Self> for Pin<Output<OpenDrain>, P, N> {
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Self, Self::Error> {
-        Ok(self)
-    }
-    fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
-        self.set_state(state);
-        Ok(self)
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Pin<Input<Floating>, P, N>, Self>
-    for Pin<Output<PushPull>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Pin<Input<Floating>, P, N>, Self::Error> {
-        Ok(self.into_floating_input())
-    }
-    fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
-        self.set_state(state);
-        Ok(self)
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Pin<Output<PushPull>, P, N>>
-    for Pin<Input<Floating>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Self, Self::Error> {
-        Ok(self)
-    }
-    fn into_output_pin(self, state: PinState) -> Result<Pin<Output<PushPull>, P, N>, Self::Error> {
-        Ok(self.into_push_pull_output_in_state(state))
-    }
-}
-
 macro_rules! gpio {
     ($GPIOX:ident, $gpiox:ident, $PEPin:ident, $port_id:expr, $PXn:ident, [
         $($PXi:ident: ($pxi:ident, $i:expr, $MODE:ty),)+
@@ -653,17 +578,17 @@ gpio!(GPIOA, gpioa, PA, 'A', PAn, [
     PA10: (pa10, 10, Input<Floating>),
     PA11: (pa11, 11, Input<Floating>),
     PA12: (pa12, 12, Input<Floating>),
-    PA13: (pa13, 13, Input<Floating>),
-    PA14: (pa14, 14, Input<Floating>),
-    PA15: (pa15, 15, Input<Floating>),
+    PA13: (pa13, 13, super::Debugger), // SWDIO, PullUp VeryHigh speed
+    PA14: (pa14, 14, super::Debugger), // SWCLK, PullDown
+    PA15: (pa15, 15, super::Debugger), // JTDI, PullUp
 ]);
 
 gpio!(GPIOB, gpiob, PB, 'B', PBn, [
     PB0: (pb0, 0, Input<Floating>),
     PB1: (pb1, 1, Input<Floating>),
     PB2: (pb2, 2, Input<Floating>),
-    PB3: (pb3, 3, Input<Floating>),
-    PB4: (pb4, 4, Input<Floating>),
+    PB3: (pb3, 3, super::Debugger), // SWO, VeryHigh speed
+    PB4: (pb4, 4, super::Debugger), // JTRST, PullUp
     PB5: (pb5, 5, Input<Floating>),
     PB6: (pb6, 6, Input<Floating>),
     PB7: (pb7, 7, Input<Floating>),
