@@ -1,9 +1,17 @@
 //! Sdio host
 
-#[allow(unused_imports)]
-use crate::gpio::{gpioa::*, gpiob::*, gpioc::*, gpiod::*, Alternate};
+#[cfg(any(
+    feature = "stm32f411",
+    feature = "stm32f412",
+    feature = "stm32f413",
+    feature = "stm32f423"
+))]
+use crate::gpio::{gpioa::*, gpiob::*};
+use crate::gpio::{gpioc::*, gpiod::*, Alternate, PushPull};
 use crate::pac::{self, RCC, SDIO};
 use crate::rcc::{Clocks, Enable, Reset};
+#[allow(unused_imports)]
+use crate::time::Hertz;
 pub use sdio_host::{
     cmd, cmd::ResponseLen, CardCapacity, CardStatus, Cmd, CurrentState, SDStatus, CIC, CID, CSD,
     OCR, RCA, SCR,
@@ -85,32 +93,32 @@ macro_rules! pins {
     feature = "stm32f479"
 ))]
 pins! {
-    CLK: [PC12<Alternate<12>>]
-    CMD: [PD2<Alternate<12>>]
-    D0: [PC8<Alternate<12>>]
-    D1: [PC9<Alternate<12>>]
-    D2: [PC10<Alternate<12>>]
-    D3: [PC11<Alternate<12>>]
+    CLK: [PC12<Alternate<PushPull, 12>>]
+    CMD: [PD2<Alternate<PushPull, 12>>]
+    D0: [PC8<Alternate<PushPull, 12>>]
+    D1: [PC9<Alternate<PushPull, 12>>]
+    D2: [PC10<Alternate<PushPull, 12>>]
+    D3: [PC11<Alternate<PushPull, 12>>]
 }
 
 #[cfg(any(feature = "stm32f412", feature = "stm32f413", feature = "stm32f423"))]
 pins! {
-    CLK: [PB15<Alternate<12>>]
-    CMD: [PA6<Alternate<12>>]
-    D0: [PB4<Alternate<12>>, PB6<Alternate<12>>]
-    D1: [PA8<Alternate<12>>]
-    D2: [PA9<Alternate<12>>]
-    D3: [PB5<Alternate<12>>]
+    CLK: [PB15<Alternate<PushPull, 12>>]
+    CMD: [PA6<Alternate<PushPull, 12>>]
+    D0: [PB4<Alternate<PushPull, 12>>, PB6<Alternate<PushPull, 12>>]
+    D1: [PA8<Alternate<PushPull, 12>>]
+    D2: [PA9<Alternate<PushPull, 12>>]
+    D3: [PB5<Alternate<PushPull, 12>>]
 }
 
 #[cfg(feature = "stm32f411")]
 pins! {
-    CLK: [PB15<Alternate<12>>]
-    CMD: [PA6<Alternate<12>>]
-    D0: [PB4<Alternate<12>>, PB7<Alternate<12>>]
-    D1: [PA8<Alternate<12>>]
-    D2: [PA9<Alternate<12>>]
-    D3: [PB5<Alternate<12>>]
+    CLK: [PB15<Alternate<PushPull, 12>>]
+    CMD: [PA6<Alternate<PushPull, 12>>]
+    D0: [PB4<Alternate<PushPull, 12>>, PB7<Alternate<PushPull, 12>>]
+    D1: [PA8<Alternate<PushPull, 12>>]
+    D2: [PA9<Alternate<PushPull, 12>>]
+    D3: [PB5<Alternate<PushPull, 12>>]
 }
 
 #[derive(Copy, Clone, Eq, PartialEq)]
@@ -149,7 +157,7 @@ pub struct Sdio {
     sdio: SDIO,
     bw: Buswidth,
     card: Option<Card>,
-    clocks: Clocks,
+    clock: Hertz,
 }
 
 /// Sd card
@@ -164,7 +172,7 @@ pub struct Card {
 
 impl Sdio {
     /// Create and enable the Sdio device
-    pub fn new<PINS: Pins>(sdio: SDIO, _pins: PINS, clocks: Clocks) -> Self {
+    pub fn new<PINS: Pins>(sdio: SDIO, _pins: PINS, clocks: &Clocks) -> Self {
         unsafe {
             //NOTE(unsafe) this reference will only be used for atomic writes with no side effects
             let rcc = &*RCC::ptr();
@@ -195,7 +203,7 @@ impl Sdio {
             sdio,
             bw: PINS::BUSWIDTH,
             card: None,
-            clocks,
+            clock: clocks.sysclk(),
         };
 
         // Make sure card is powered off
@@ -308,7 +316,7 @@ impl Sdio {
         });
 
         // Wait for 2 ms after changing power settings
-        cortex_m::asm::delay(2 * (self.clocks.sysclk().0 / 1000));
+        cortex_m::asm::delay(2 * (self.clock.0 / 1000));
     }
 
     /// Get a reference to the initialized card
@@ -331,12 +339,14 @@ impl Sdio {
         self.cmd(cmd::read_single_block(blockaddr))?;
 
         let mut i = 0;
-        let mut sta;
 
-        while {
-            sta = self.sdio.sta.read();
-            sta.rxact().bit_is_set()
-        } {
+        let status = loop {
+            let sta = self.sdio.sta.read();
+
+            if sta.rxact().bit_is_clear() {
+                break sta;
+            }
+
             if sta.rxfifohf().bit() {
                 for _ in 0..8 {
                     let bytes = self.sdio.fifo.read().bits().to_le_bytes();
@@ -346,11 +356,11 @@ impl Sdio {
             }
 
             if i == block.len() {
-                break;
+                break sta;
             }
-        }
+        };
 
-        status_to_error(sta)?;
+        status_to_error(status)?;
 
         // Wait for card to be ready
         while !self.card_ready()? {}
@@ -373,12 +383,14 @@ impl Sdio {
         self.cmd(cmd::write_single_block(blockaddr))?;
 
         let mut i = 0;
-        let mut sta;
 
-        while {
-            sta = self.sdio.sta.read();
-            sta.txact().bit_is_set()
-        } {
+        let status = loop {
+            let sta = self.sdio.sta.read();
+
+            if sta.txact().bit_is_clear() {
+                break sta;
+            }
+
             if sta.txfifohe().bit() {
                 for _ in 0..8 {
                     let mut wb = [0u8; 4];
@@ -390,12 +402,20 @@ impl Sdio {
             }
 
             if i == block.len() {
+                break sta;
+            }
+        };
+
+        status_to_error(status)?;
+
+        // Wait for SDIO module to finish transmitting data
+        loop {
+            let sta = self.sdio.sta.read();
+            if !sta.txact().bit_is_set() {
                 break;
             }
         }
 
-        status_to_error(sta)?;
-
         // Wait for card to finish writing data
         while !self.card_ready()? {}
 
@@ -409,10 +429,16 @@ impl Sdio {
         assert!(block_size <= 14);
 
         // Command AND Data state machines must be idle
-        while self.sdio.sta.read().cmdact().bit_is_set()
-            || self.sdio.sta.read().rxact().bit_is_set()
-            || self.sdio.sta.read().txact().bit_is_set()
-        {}
+        loop {
+            let status = self.sdio.sta.read();
+
+            if status.cmdact().bit_is_clear()
+                && status.rxact().bit_is_clear()
+                && status.txact().bit_is_clear()
+            {
+                break;
+            }
+        }
 
         let dtdir = if card_to_controller {
             DTDIR_A::CARDTOCONTROLLER
@@ -459,12 +485,14 @@ impl Sdio {
 
         let mut status = [0u32; 16];
         let mut idx = 0;
-        let mut sta;
 
-        while {
-            sta = self.sdio.sta.read();
-            sta.rxact().bit_is_set()
-        } {
+        let s = loop {
+            let sta = self.sdio.sta.read();
+
+            if sta.rxact().bit_is_clear() {
+                break sta;
+            }
+
             if sta.rxfifohf().bit() {
                 for _ in 0..8 {
                     status[15 - idx] = self.sdio.fifo.read().bits().swap_bytes();
@@ -473,11 +501,11 @@ impl Sdio {
             }
 
             if idx == status.len() {
-                break;
+                break sta;
             }
-        }
+        };
 
-        status_to_error(sta)?;
+        status_to_error(s)?;
         Ok(SDStatus::from(status))
     }
 
@@ -499,23 +527,25 @@ impl Sdio {
 
         let mut buf = [0; 2];
         let mut i = 0;
-        let mut sta;
 
-        while {
-            sta = self.sdio.sta.read();
-            sta.rxact().bit_is_set()
-        } {
+        let status = loop {
+            let sta = self.sdio.sta.read();
+
+            if sta.rxact().bit_is_clear() {
+                break sta;
+            }
+
             if sta.rxdavl().bit() {
                 buf[1 - i] = self.sdio.fifo.read().bits().swap_bytes();
                 i += 1;
             }
 
             if i == 2 {
-                break;
+                break sta;
             }
-        }
+        };
 
-        status_to_error(sta)?;
+        status_to_error(status)?;
         Ok(SCR::from(buf))
     }
 
@@ -583,33 +613,44 @@ impl Sdio {
 
         let mut timeout: u32 = 0xFFFF_FFFF;
 
-        let mut sta;
-        if cmd.response_len() == ResponseLen::Zero {
+        let status = if cmd.response_len() == ResponseLen::Zero {
             // Wait for command sent or a timeout
-            while {
-                sta = self.sdio.sta.read();
+            loop {
+                let sta = self.sdio.sta.read();
+
+                if sta.cmdact().bit_is_clear()
+                    && (sta.ctimeout().bit_is_set() || sta.cmdsent().bit_is_set())
+                {
+                    break sta;
+                }
+
+                if timeout == 0 {
+                    return Err(Error::SoftwareTimeout);
+                }
 
-                (!(sta.ctimeout().bit() || sta.cmdsent().bit()) || sta.cmdact().bit_is_set())
-                    && timeout > 0
-            } {
                 timeout -= 1;
             }
         } else {
-            while {
-                sta = self.sdio.sta.read();
-                (!(sta.ctimeout().bit() || sta.cmdrend().bit() || sta.ccrcfail().bit())
-                    || sta.cmdact().bit_is_set())
-                    && timeout > 0
-            } {
+            loop {
+                let sta = self.sdio.sta.read();
+
+                if sta.cmdact().bit_is_clear()
+                    && (sta.ctimeout().bit()
+                        || sta.cmdrend().bit_is_set()
+                        || sta.ccrcfail().bit_is_set())
+                {
+                    break sta;
+                }
+
+                if timeout == 0 {
+                    return Err(Error::SoftwareTimeout);
+                }
+
                 timeout -= 1;
             }
-        }
+        };
 
-        if timeout == 0 {
-            return Err(Error::SoftwareTimeout);
-        }
-
-        status_to_error(sta)
+        status_to_error(status)
     }
 }
 
